@@ -1,22 +1,18 @@
-__version__ = '0.45.1'
+__version__ = '0.49.0'
 
-from json import loads, JSONDecodeError
-from typing import TypedDict as _TypedDict
 from functools import partial as _partial
-from re import compile as _rc
-# noinspection PyUnresolvedReferences
-from re import findall as _findall
-# noinspection PyUnresolvedReferences
-from io import BytesIO as _BytesIO, StringIO as _StringIO
+from json import JSONDecodeError, loads
 from logging import error
+from re import compile as _rc, findall as _findall
+from typing import TypedDict as _TypedDict
+from warnings import warn as _warn
 
+from aiohttp import (
+    ClientSession as _ClientSession,
+    ClientTimeout as _ClientTimeout,
+)
 from jdatetime import datetime as _jdatetime
-from pandas import read_csv as _read_csv, DataFrame as _DataFrame
-# noinspection PyUnresolvedReferences
-from pandas import to_numeric as _to_numeric, read_html as _read_html
-from aiohttp import ClientSession as _ClientSession, \
-    ClientTimeout as _ClientTimeout
-
+from pandas import DataFrame as _DataFrame, read_csv as _read_csv
 
 _csv2df = _partial(_read_csv, low_memory=False, engine='c', lineterminator=';')
 _F = r'(-?\d+(?:\.\d+)?)'  # float pattern
@@ -110,7 +106,7 @@ class Session:
     def __new__(cls, *args, **kwargs) -> _ClientSession:
         global SESSION
         if 'timeout' not in kwargs:
-            kwargs['timeout'] = _ClientTimeout(total=30., sock_connect=10., sock_read=10.)
+            kwargs['timeout'] = _ClientTimeout(total=30., sock_connect=30., sock_read=30.)
         SESSION = _ClientSession(**kwargs)
         return SESSION
 
@@ -123,9 +119,10 @@ _HEADERS = {
 
 # this function should only be called from _get below
 async def _session_get(url: str) -> bytes:
-    return await (await SESSION.get(
-        url, headers=_HEADERS
-    )).read()
+    r = await SESSION.get(url, headers=_HEADERS)
+    if r.history:
+        _warn(f'r.history is not empty (possible redirection): {r.history}')
+    return await r.read()
 
 
 async def _get(url: str, *, fa=False) -> str | bytes:
@@ -135,7 +132,7 @@ async def _get(url: str, *, fa=False) -> str | bytes:
     return content
 
 
-_DOMAIN = 'http://tsetmc.com/'
+_DOMAIN = 'http://old.tsetmc.com/'
 # API does not work on www domain
 _API = 'http://cdn.tsetmc.com/api/'
 
@@ -148,8 +145,8 @@ async def _get_par_tree(path: str, *, fa=True) -> str | bytes:
     return await _get(f'{_DOMAIN}Loader.aspx?ParTree={path}', fa=fa)
 
 
-async def _api(path: str):
-    content = await _get(f'{_API}{path}')
+async def _api(path: str, *, fa=False):
+    content = await _get(f'{_API}{path}', fa=fa)
     try:
         return loads(content)
     except JSONDecodeError:
