@@ -1,10 +1,10 @@
-from pandas import concat as _concat, read_json as _read_json, NA as _NA
+from io import StringIO as _StringIO
+from typing import TypedDict as _TypedDict
+
 from bs4 import BeautifulSoup as _BeautifulSoup
+from pandas import NA as _NA, concat as _concat, read_html as _read_html
 
-from tsetmc import _get, _get_par_tree, _numerize, _read_html, _DataFrame, \
-    _StringIO, \
-    _partial
-
+from tsetmc import _api, _DataFrame, _get_par_tree, _numerize, _partial
 
 _make_soup = _partial(_BeautifulSoup, features='lxml')
 
@@ -41,14 +41,15 @@ async def industrial_groups_overview() -> _DataFrame:
     return df
 
 
-async def market_map_data() -> _DataFrame:
-    text = await _get('http://new.tsetmc.com/weatherforecast', fa=True)
-    df: _DataFrame = _read_json(_StringIO(text), convert_dates=False)
-    comma_nums = ('QTotCap', 'QTotTran5J', 'ZTotTran')
-    for c in comma_nums:
-        df[c] = df[c].str.replace(
-            ',', '', regex=False
-        ).astype('int64', copy=False)
+async def market_map_data(*, market=0, size=9999, sector=0, typeSelected=1, heven=0) -> _DataFrame:
+    j = await _api(
+        f'ClosingPrice/GetMarketMap'
+        f'?market={market}&size={size}&sector={sector}'
+        f'&typeSelected={typeSelected}&hEven={heven}',
+        fa=True,
+    )
+    df = _DataFrame(j)
+    df['insCode'] = df.insCode.astype('int64')
     return df
 
 
@@ -97,3 +98,32 @@ def _parse_tds(tds):
         except ValueError:
             if td == '\xa0':
                 yield _NA
+
+
+class _MarketOverview(_TypedDict):
+    lastDataDEven: int
+    lastDataHEven: int
+    indexLastValue: float
+    indexChange: float
+    indexEqualWeightedLastValue: float
+    indexEqualWeightedChange: float
+    marketActivityDEven: int
+    marketActivityHEven: int
+    marketActivityZTotTran: int
+    marketActivityQTotCap: float
+    marketActivityQTotTran: float
+    marketState: str
+    marketValue: float
+    marketValueBase: float
+    marketStateTitle: str
+
+
+async def market_overview(n=1) -> _MarketOverview:
+    """Return GetMarketOverview result.
+
+    :param n:
+        1: bourse
+        2: fara-bourse
+    """
+    j = await _api(f'MarketData/GetMarketOverview/{n}')
+    return j['marketOverview']
