@@ -1,18 +1,37 @@
 from ast import literal_eval as _literal_eval
 from datetime import datetime as _datetime
 from functools import partial as _partial
+from io import BytesIO as _BytesIO, StringIO as _StringIO
 from logging import warning as _warning
 from pathlib import Path
 from re import fullmatch as _fullmatch
+from warnings import warn as _warn
 
-from pandas import to_datetime as _to_datetime, read_csv as _read_csv
-
-from . import _FARSI_NORM, _MarketState, _api, _csv2df, _F, _TypedDict, \
-    _get_data, \
-    _numerize, _parse_market_state, _parse_ombud_messages, _rc, \
-    _get, _StringIO, _BytesIO, _DataFrame, \
-    _to_numeric, _read_html, _findall, _jstrptime, _get_par_tree, _jdatetime
+from pandas import (
+    read_csv as _read_csv,
+    read_html as _read_html,
+    to_datetime as _to_datetime,
+)
 
+from . import (
+    _F,
+    _FARSI_NORM,
+    _api,
+    _csv2df,
+    _DataFrame,
+    _findall,
+    _get,
+    _get_data,
+    _get_par_tree,
+    _jdatetime,
+    _jstrptime,
+    _MarketState,
+    _numerize,
+    _parse_market_state,
+    _parse_ombud_messages,
+    _rc,
+    _TypedDict,
+)
 
 _strptime = _datetime.strptime
 _j_ymd_parse = _partial(_jstrptime, format='%Y/%m/%d')
@@ -112,11 +131,148 @@ class _LiveData(_TypedDict, total=False):
     nav_datatime: _jdatetime
 
 
+class _ETF(_TypedDict):
+    insCode: str
+    deven: int
+    hEven: int
+    pRedTran: int
+    pSubTran: int
+    iClose: int
+
+
+class _InstrumentState(_TypedDict):
+    idn: int
+    dEven: int
+    hEven: int
+    insCode: None
+    cEtaval: str
+    realHeven: int
+    underSupervision: int
+    cEtavalTitle: str
+
+
+class _ClosingPriceInfo(_TypedDict):
+    instrumentState: _InstrumentState
+    instrument: None
+    lastHEven: int
+    finalLastDate: int
+    nvt: float
+    mop: int
+    thirtyDayClosingHistory: None
+    priceChange: float
+    priceMin: float
+    priceMax: float
+    priceYesterday: float
+    priceFirst: float
+    last: bool
+    id: int
+    insCode: str
+    dEven: int
+    hEven: int
+    pClosing: float
+    iClose: bool
+    yClose: bool
+    pDrCotVal: float
+    zTotTran: float
+    qTotTran5J: float
+    qTotCap: float
+
+class _ClientType(_TypedDict):
+    buy_I_Volume: float
+    buy_N_Volume: float
+    buy_DDD_Volume: float
+    buy_CountI: int
+    buy_CountN: int
+    buy_CountDDD: int
+    sell_I_Volume: float
+    sell_N_Volume: float
+    sell_CountI: int
+    sell_CountN: int
+
+
+class _Sector(_TypedDict):
+    dEven: int
+    cSecVal: str
+    lSecVal: str
+
+
+class _SubSector(_TypedDict):
+    dEven: int
+    cSecVal: None
+    cSoSecVal: int
+    lSoSecVal: str
+
+
+class _Identity(_TypedDict):
+    sector: _Sector
+    subSector: _SubSector
+    cValMne: str
+    lVal18: str
+    cSocCSAC: str
+    lSoc30: str
+    yMarNSC: str
+    yVal: str
+    insCode: str
+    lVal30: str
+    lVal18AFC: str
+    flow: int
+    cIsin: str
+    zTitad: float
+    baseVol: int
+    instrumentID: str
+    cgrValCot: str
+    cComVal: str
+    lastDate: int
+    sourceID: int
+    flowTitle: str
+    cgrValCotTitle: str
+
+
+class _ShareHolder(_TypedDict):
+    shareHolderID: int
+    shareHolderName: str | None  # None: reserved code of ETFs
+    cIsin: str
+    dEven: int
+    numberOfShares: float
+    perOfShares: float
+    change: int
+    changeAmount: float
+
+
+class _Message(_TypedDict):
+    tseMsgIdn: int
+    dEven: int
+    hEven: int
+    tseTitle: str
+    tseDesc: str
+    flow: int
+
+
+class _Codal(_TypedDict):
+    id: int
+    symbol: str
+    name: str
+    title: str
+    sentDateTime_Gregorian: str
+    publishDateTime_Gregorian: str
+    publishDateTime_DEven: int
+    mainTableRowID: int
+    hasHtmlReport: int
+    hasExcelReport: int
+    hasPDFReport: int
+    hasXMLReport: int
+    attachmentID: int
+    contentType: int
+    fileName: str
+    fileExtension: str
+    tracingNo: str
+
+
 class Instrument:
 
-    __slots__ = 'code', '_l18', '_l30', '_cisin'
+    __slots__ = 'code', '_l18', '_l30', '_cisin', '_cs'
 
-    def __init__(self, code: int, l18: str = None, l30: str = None):
+    def __init__(self, code: str | int, l18: str = None, l30: str = None):
         self.code = code
         self._l18 = l18
         self._l30 = l30
@@ -142,7 +298,7 @@ class Instrument:
         try:
             self._l18, self._l30 = _l18_l30(self.code)
         except KeyError:
-            await self.page_data()
+            await self.info()
         return self._l18
 
     @property
@@ -156,7 +312,7 @@ class Instrument:
         try:
             self._l18, self._l30 = _l18_l30(self.code)
         except KeyError:
-            await self.page_data()
+            await self.info()
         return self._l30
 
     @property
@@ -164,12 +320,17 @@ class Instrument:
         try:
             return self._cisin
         except AttributeError:
-            # can also be fetched using self.identification()
-            # but that won't load self._l18 since 'نماد فارسی'
-            # sometimes contains descriptions like "وسديد - لغو پذیرش شده".
-            await self.page_data()
+            await self.info()
         return self._cisin
 
+    @property
+    async def cs(self) -> str:
+        try:
+            return self._cs
+        except AttributeError:
+            await self.info()
+        return self._cs
+
     @staticmethod
     async def from_l18(l18: str, /) -> 'Instrument':
         try:
@@ -178,6 +339,75 @@ class Instrument:
             return await Instrument.from_search(l18)
         return Instrument(ins_code, l18, l30)
 
+    async def info(self) -> dict:
+        j = await _api(f'Instrument/GetInstrumentInfo/{self.code}', fa=True)
+        d = j['instrumentInfo']
+        # cache for properties
+        self._cs = d['sector']['cSecVal']
+        self._cisin = d['cIsin']
+        self._l18 = d['lVal18AFC']
+        self._l30 = d['lVal30']
+        return j['instrumentInfo']
+
+    async def trades(self) -> _DataFrame:
+        j = await _api(f'Trade/GetTrade/{self.code}')
+        df = _DataFrame(j['trade'], copy=False)
+        return df
+
+    async def codal(self, n=9) -> list[_Codal]:
+        j = await _api(f'Codal/GetPreparedDataByInsCode/{n}/{self.code}', fa=True)
+        return j['preparedData']
+
+    async def daily_closing_price(self, n=9) -> _DataFrame:
+        j = await _api(f'ClosingPrice/GetClosingPriceDailyList/{self.code}/{n}')
+        df = _DataFrame(j['closingPriceDaily'], copy=False)
+        df['date'] = _to_datetime(
+            df.pop('dEven').astype(str) + df.pop('hEven').astype(str)
+        )
+        df.set_index('date', inplace=True)
+        return df
+
+    async def closing_price_info(self) -> _ClosingPriceInfo:
+        j = await _api(f'ClosingPrice/GetClosingPriceInfo/{self.code}', fa=True)
+        return j['closingPriceInfo']
+
+    async def best_limits(self) -> _DataFrame:
+        j = await _api(f'BestLimits/{self.code}')
+        df = _DataFrame(j['bestLimits'], copy=False)
+        return df
+
+    async def client_type(self) -> _ClientType:
+        j = await _api(f'ClientType/GetClientType/{self.code}/1/0')
+        return j['clientType']
+    
+    async def etf(self) -> _ETF:
+        """Return ETF data. (Includes redemption NAV and datetime of it).
+
+        This method is only valid for ETFs.
+        """
+        j = await _api(f'Fund/GetETFByInsCode/{self.code}')
+        return j['etf']
+
+    async def related_companies(self, cs=None) -> dict[str, _DataFrame]:
+        if cs is None:
+            cs = await self.cs
+
+        j = await _api(f'ClosingPrice/GetRelatedCompany/{cs}')
+
+        j['relatedCompany'] = _DataFrame(
+            # flatten the records
+            [c.pop('instrument') | c for c in j.pop('relatedCompany')],
+            copy=False,
+        )
+
+        j['relatedCompanyThirtyDayHistory'] = _DataFrame(
+            j.pop('relatedCompanyThirtyDayHistory'),
+            copy=False,
+        )
+
+        return j
+
+
     async def page_data(
         self, general=True, trade_history=False, related_companies=False
     ) -> dict:
@@ -192,7 +422,17 @@ class Instrument:
 
         For the meaning of other column names see:
             http://www.tsetmc.com/Site.aspx?ParTree=151713
+
+        Alternative methods from the new API:
+        - self.info
+        - self.related_companies
+        - self.daily_closing_price
         """
+        _warn(
+            '`Instrument.page_data` is deprecated; see its doc-string for alternatives',
+            DeprecationWarning, stacklevel=2,
+        )
+
         text = await _get_par_tree(f'151311&i={self.code}')
         if general:
             m = _PAGE_VARS(text)
@@ -204,10 +444,11 @@ class Instrument:
             l30 = self._l30 = title_match[1]
             l18 = self._l18 = m['LVal18AFC']
             cisin = self._cisin = m['CIsin']
+            cs = self._cs = m['CSecVal']
             result = {
                 'bvol': int(m['BaseVol']),
                 'cisin': cisin,
-                'cs': int(m['CSecVal']),
+                'cs': int(cs),
                 'eps': int(eps) if eps else None,
                 'sps': float(sps) if sps else None,
                 'flow': int(m['Flow']),
@@ -244,7 +485,6 @@ class Instrument:
             result['related_companies'] = [
                 Instrument(code, l18, l30) for (code, l18, l30) in
                 _literal_eval(_STR_TO_NUM(m[1]))]
-        # todo: add 'codal_data'
         return result
 
     async def live_data(
@@ -254,7 +494,17 @@ class Instrument:
 
         :keyword best_limits: parse best_limits and include related values.
         :keyword market_state: parse values related to market state.
+
+        Alternatives from the new API:
+        - `Instrument.closing_price_info`
+        - `Instrument.best_limits`
+        - `Instrument.etf`
+        - `general.market_overview`
         """
+        _warn(
+            '`Instrument.live_data` is deprecated; see its doc-string for alternatives',
+            DeprecationWarning, stacklevel=2,
+        )
         # apparently, http://www.tsetmc.com/tsev2/data/instinfodata.aspx?i=...
         # and http://www.tsetmc.com/tsev2/data/instinfofast.aspx?i=...
         # return the same response.
@@ -263,15 +513,14 @@ class Instrument:
             # &e=1 parameter is required to get cancel NAV for ETFs but it 
             # seems to be ignored if no NAV is defined for the instrument.
             # &c= is normally set to CSecVal, but does not seem to be required.
-            # CSecVal is idustry group code:
+            # CSecVal is industry group code:
             # http://redirectcdn.tsetmc.com/Site.aspx?ParTree=111411111B&LnkIdn=107
             f'?i={self.code}&c=&e=1', fa=True)
         # the _s are unknown
         try:
             price_info, index_info, orders_info, _, _, _, group_info, _, _ = text.split(';')
         except ValueError:
-            # todo: fix not enough values to unpack
-            _warning(text)  # The service is unavailable.
+            _warning(text)  # usually means the service is unavailable
             raise
         if general:
             result = _parse_price_info(price_info)
@@ -295,6 +544,10 @@ class Instrument:
 
         Use :meth:`Instrument.on_date(<date>).trades` for intraday trades.
         """
+        _warn(
+            '`Instrument.trade_history` is deprecated; use `Instrument.daily_closing_price` instead.',
+            DeprecationWarning, stacklevel=2,
+        )
         content = await _get_data(f'InstTradeHistory.aspx?i={self.code}&Top={top}&A={all_:d}')
         df = _csv2df(
             _BytesIO(content)
@@ -305,25 +558,36 @@ class Instrument:
         return df
 
     async def price_history(self, adjusted: bool = True) -> _DataFrame:
+        # As far as I can thell the new tsetmc site does not have any
+        # API for adjusted price history, but see self.price_adjustments.
         content = await _get(
-            f'http://members.tsetmc.com/tsev2/chart/data/Financial.aspx?i={self.code}&t=ph&a={adjusted:d}')
+            f'https://members.tsetmc.com/tsev2/chart/data/Financial.aspx?i={self.code}&t=ph&a={adjusted:d}')
         df = _csv2df(
             _BytesIO(content)
             , names=('date', 'pmax', 'pmin', 'pf', 'pl', 'tvol', 'pc')
             , index_col='date', parse_dates=True)
         return df
 
-    async def client_type(self) -> _DataFrame:
+
+    async def client_type_history_old(self) -> _DataFrame:
         """Get daily natural/legal history.
 
         In column names `n_` prefix stands for natural and `l_` for legal.
 
         This method returns the information available at the "حقیقی-حقوقی" tab
-        of the instrument. It uses the `clienttype.aspx` module.
+        of the instrument. It uses the `clienttype.aspx` module of
+        old.tsetmc.com.
+
+        This method may stop working in the future if the old tsetmc site
+        shuts down.
 
         See also:
             :meth:`Instrument.client_type_history`
         """
+        _warn(
+            '`Instrument.client_type_history_old` is deprecated; use `Instrument.client_type_history` instead.',
+            DeprecationWarning, stacklevel=2,
+        )
         return _csv2df(
             _BytesIO(await _get_data(f'clienttype.aspx?i={self.code}'))
             , names=(
@@ -331,21 +595,27 @@ class Instrument:
                 , 'n_buy_count', 'l_buy_count', 'n_sell_count', 'l_sell_count'
                 , 'n_buy_volume', 'l_buy_volume', 'n_sell_volume', 'l_sell_volume'
                 , 'n_buy_value', 'l_buy_value', 'n_sell_value', 'l_sell_value')
-            , index_col='date', parse_dates=True , dtype='uint64')
+            , index_col='date', parse_dates=True , dtype='int64')
 
-    async def client_type_history(self, date: int | str = None) -> _DataFrame | dict:
+
+    async def client_type_history(
+        self, date: int | str = None
+    ) -> _DataFrame | dict:
         """Return natural/legal client type history.
 
         :param date: Gregorian date in YYYYMMDD format. If None, return the
-        full history as a DataFrame. Otherwise, return the data for that
-        specific date as a dict.
+            full history as a DataFrame. Otherwise, return the data for that
+            specific date as a dict.
 
-        Uses the information from api/ClientType/GetClientTypeHistory.
+        Legal persoans are indicated with `N`, natural persons with `I`.
 
         See also:
             :meth:`Instrument.client_type`
         """
         if date is None:
+            # the following api call returns a result that is equivalent to
+            # j = (await _api(f'ClientType/GetClientType/{self.code}/1/0'))[
+            #     'clientType']
             j = await _api(f'ClientType/GetClientTypeHistory/{self.code}')
             return _DataFrame(j['clientType'], copy=False)
 
@@ -358,23 +628,63 @@ class Instrument:
         Related API descriptions:
             https://cdn.tsetmc.com/Site.aspx?ParTree=1114111118&LnkIdn=83
             http://en.tsetmc.com/Site.aspx?ParTree=111411111Z
+
+        This method uses old.tsetmc.com.
+        For the new API use `Instrument.identity`.
         """
+        _warn(
+            '`Instrument.identification` is deprecated; use `Instrument.identity` instead.',
+            DeprecationWarning, stacklevel=2,
+        )
         text = await _get_par_tree(f'15131M&i={self.code}')
         df = _read_html(text)[0]
         return dict(zip(df[0], df[1]))
 
+    async def identity(self) -> _Identity:
+        j = await _api(f'Instrument/GetInstrumentIdentity/{self.code}', fa=True)
+        return j['instrumentIdentity']
+
     async def introduction(self) -> dict[str, str]:
         """Return the information available in introduction (معرفی) tab."""
+        _warn(
+            '`Instrument.introduction` is deprecated; use `Instrument.publisher` instead.',
+            DeprecationWarning, stacklevel=2,
+        )
         text = await _get_par_tree(
             f'15131V&s={await self._arabic_l18}')
         df = _read_html(text)[0]
         return dict(zip(df[0].str.removesuffix(' :'), df[1]))
 
+    async def publisher(self) -> dict:
+        j = await _api(f'Codal/GetCodalPublisherBySymbol/{await self._arabic_l18}', fa=True)
+        return j['codalPublisher']
+
     @staticmethod
     async def from_search(s: str) -> 'Instrument':
-        """`search(s)` and return the first result as Instrument."""
-        l18, l30, ins_code = (await search(s)).iloc[0][:3]
-        return Instrument(ins_code, l18, l30)
+        """`Search for `s` and return the first result as an Instrument."""
+        d =  (await search(s))[0]
+        return Instrument(d['insCode'], d['lVal18AFC'], d['lVal30'])
+
+    async def share_holders(self) -> list[_ShareHolder]:
+        """Return a list of the current major holders of this instrument."""
+        j = await _api(f'Shareholder/GetInstrumentShareHolderLast/{self.code}', fa=True)
+        return j['shareHolder']
+
+    async def share_holder_history(
+        self, share_holder_id: int,
+        days: int=90,
+    ) -> list[_ShareHolder]:
+        """Return history of share changes of a particular holder.
+
+        Obtain `share_holder_id` from `self.share_holders`.
+        To get other companies of the same shareholder use the
+        `share_holder_companies` function.
+        """
+        j = await _api(
+            f'Shareholder/GetShareHolderHistory/{self.code}/{share_holder_id}/{days}',
+            fa=True
+        )
+        return j['shareHolder']
 
     async def holders(self, cisin=None) -> _DataFrame:
         """Get list of current major unit/shareholders.
@@ -384,6 +694,10 @@ class Instrument:
         See also:
             :meth:`Instrument.on_date(<date>).holders`
         """
+        _warn(
+            '`Instrument.holders` is deprecated; use `Instrument.share_holders` instead.',
+            DeprecationWarning, stacklevel=2,
+        )
         if cisin is None:
             cisin = await self.cisin
         text = await _get_par_tree(f'15131T&c={cisin}')
@@ -407,6 +721,10 @@ class Instrument:
         If both `history` and `other_holdings` are True, then a tuple of
         DataFrames will be returned.
         """
+        _warn(
+            '`Instrument.holder` is deprecated; use `Instrument.share_holder_history` or `instruments.share_holder_companies` instead.',
+            DeprecationWarning, stacklevel=2,
+        )
         text = await _get_data(f'ShareHolder.aspx?i={id_cisin}', fa=True)
         hist, _, oth = text.partition('#')
 
@@ -414,7 +732,7 @@ class Instrument:
             return _csv2df(
                 _StringIO(hist),
                 names=('date', 'shares'),
-                dtype='uint64',
+                dtype='int64',
                 index_col='date',
                 parse_dates=True)
 
@@ -431,154 +749,34 @@ class Instrument:
         else:
             return other_holdings_df()
 
-    async def intraday(
-        self, date: int | str, *,
-        general=False,
-        thresholds=False,
-        closings=False,
-        candles=False,
-        states=True,
-        trades=True,
-        holders=False,
-        yesterday_holders=False,
-        client_types=True,
-        best_limits=True,
-    ) -> _IntraDay:
-        """Get intraday info for the given date in YYYYMMDD format.
-
-        For the meaning of instrument state codes refer to
-            http://en.tsetmc.com/Site.aspx?ParTree=111411111Y
-
-        This method uses the old way of fetching intraday data from tsetmc.com
-        which is not used on the live site anymore.
-
-        For fetching individual parameters, use :meth:`instrument.on_date`.
-        """
-        text = await _get_par_tree(f'15131P&i={self.code}&d={date}')
-        find = text.find
-        find_start = 0
-        result = {}
-        if general:
-            find_start = find('InstSimpleData=') + 15
-            end = find('];', find_start)
-            result['general'] = dict(zip(
-                ('l30', 'l18', 'market', 'flow_name', 'flow', 'group_code', 'cisin', 'isin', 'z', 'bvol'),
-                _literal_eval(text[find_start: end + 1])))
-            find_start = end
-        if thresholds:
-            find_start = find('StaticTreshholdData=') + 20
-            end = find('];', find_start)
-            result['thresholds'] = _DataFrame(
-                _literal_eval(text[find_start: end + 1]),
-                columns=('time', 'tmax', 'tmin'),
-                copy=False,
-            )
-            find_start = end
-        if closings:
-            find_start = find('ClosingPriceData=', find_start) + 17
-            end = find('];', find_start)
-            evaluated = _literal_eval(text[find_start: end + 1])
-            closings = _DataFrame(evaluated, columns=(
-                'date', '?1', 'pl', 'pc', 'pf', 'py', 'pmin', 'pmax', 'tno',
-                'tvol', 'tval', '?2', 'heven'), copy=False)
-            if len(closings['?1'].unique()) != 1 or closings['?2'].unique() != 1:
-                # See if you can find the meaning of ?2 column
-                _warning(f'Unusual ?1 or ?2. Parameters: {date=} {self.code}. Please report this at https://github.com/5j9/tsetmc/issues.')
-            result['closings'] = closings
-            find_start = end
-        if candles:
-            find_start = find('IntraDayPriceData=', find_start) + 18
-            end = find('];', find_start)
-            evaluated = _literal_eval(text[find_start: end + 1])
-            result['candles'] = _DataFrame(
-                evaluated,
-                columns=('time', 'high', 'low', 'open', 'close', 'tvol'),
-                copy=False,
-            )
-            find_start = end
-        if states:
-            find_start = find('InstrumentStateData=', find_start) + 20
-            end = find('];', find_start)
-            evaluated = _literal_eval(text[find_start: end + 1])
-            result['states'] = _DataFrame(
-                evaluated, columns=('date', 'time', 'state'), copy=False
-            )
-            find_start = end
-        if trades:
-            find_start = find('IntraTradeData=', find_start) + 15
-            end = find('];', find_start)
-            evaluated = _literal_eval(text[find_start: end + 1])
-            find_start = end
-            trades = _DataFrame(
-                evaluated,
-                columns=('-', 'time', 'tvol', 'pl', 'annulled'),
-                copy=False,
-            )
-            trades['annulled'] = trades['annulled'].astype(bool, False)
-            numeric_cols = ['-', 'tvol', 'pl']
-            trades[numeric_cols] = trades[numeric_cols].apply(_to_numeric)
-            trades.set_index('-', inplace=True)
-            trades.sort_index(inplace=True)
-            result['trades'] = trades
-        if holders or yesterday_holders:
-            holder_cols = ('id', 'cisin', 'shares', 'percent', 'change', 'name')
-        if holders:
-            find_start = find('ShareHolderData=', find_start) + 16
-            end = find('];', find_start)
-            evaluated = _literal_eval(text[find_start: end + 1])
-            # noinspection PyUnboundLocalVariable
-            result['holders'] = _DataFrame(
-                evaluated, columns=holder_cols, copy=False
-            )
-            find_start = end
-        if yesterday_holders:
-            find_start = find('ShareHolderDataYesterday=', find_start) + 25
-            end = find('];', find_start)
-            evaluated = _literal_eval(text[find_start: end + 1])
-            result['yesterday_holders'] = _DataFrame(
-                evaluated, columns=holder_cols, copy=False
-            )
-            find_start = end
-        if client_types:
-            find_start = find('ClientTypeData=', find_start) + 15
-            end = find('];', find_start)
-            evaluated = _literal_eval(text[find_start: end + 1])
-            result['client_types'] = dict(zip((
-                'n_buy_count', 'l_buy_count',
-                'n_sell_count', 'l_sell_count',
-                'n_buy_volume', 'l_buy_volume',
-                'n_sell_volume', 'l_sell_volume',
-                'n_buy_percent', 'l_buy_percent',
-                'n_sell_percent', 'l_sell_percent',
-                'n_buy_value', 'l_buy_value',
-                'n_sell_value', 'l_sell_value',
-                'n_mean_buy_price', 'l_mean_buy_price',
-                'n_mean_sell_price', 'l_mean_sell_price',
-            ), evaluated))
-            find_start = end
-        if best_limits:
-            find_start = find('var BestLimitData=', find_start) + 18
-            end = find('];', find_start)
-            evaluated = _literal_eval(text[find_start: end + 1])
-            best_limits_df = _DataFrame(
-                evaluated,
-                columns=('time', 'row', 'zd', 'qd', 'pd', 'po', 'qo', 'zo'),
-                copy=False,
-            )
-            # todo: use astype?
-            best_limits_df : _DataFrame = best_limits_df.apply(_to_numeric)
-            best_limits_df.set_index('time', inplace=True)
-            result['best_limits'] = best_limits_df
-        return result
-
     async def adjustments(self) -> _DataFrame:
+        _warn(
+            '`Instrument.adjustments` is deprecated; use `Instrument.price_adjustments` instead.',
+            DeprecationWarning, stacklevel=2,
+        )
         text = await _get_par_tree(f'15131G&i={self.code}', fa=False)
         df = _read_html(text)[0]
         df.columns = ('date', 'adj_pc', 'pc')
         df['date'] = df['date'].apply(_j_ymd_parse)
         return df
 
+
+    async def price_adjustments(self) -> _DataFrame:
+        j = await _api(f'ClosingPrice/GetPriceAdjustList/{self.code}')
+        df = _DataFrame(j['priceAdjust'], copy=False)
+        df['dEven'] = _to_datetime(df['dEven'], format='%Y%m%d')
+        df.set_index('dEven', inplace=True)
+        return df
+
+    async def messages(self) -> list[_Message]:
+        j = await _api(f'Msg/GetMsgByInsCode/{self.code}', fa=True)
+        return j['msg']
+
     async def ombud_messages(self) -> _DataFrame:
+        _warn(
+            '`Instrument.ombud_messages` is deprecated; use `Instrument.messages` instead.',
+            DeprecationWarning, stacklevel=2,
+        )
         return _parse_ombud_messages(await _get_par_tree(f'15131W&i={self.code}'))
 
     async def dps_history(self) -> _DataFrame:
@@ -586,6 +784,8 @@ class Instrument:
 
         :raises pandas.errors.EmptyDataError: No columns to parse from file
         """
+        # Note: Currently does not have an _api equivalent. The DPS tab is
+        # non-functional in the new tsetmc website.
         content = await _get_data(f'DPSData.aspx?s={await self._arabic_l18}')
         df = _csv2df(
             _BytesIO(content),
@@ -615,6 +815,43 @@ class Instrument:
         return InstrumentOnDate(_inst=self, _date=date)
 
 
+class _ClosingPrice(_TypedDict):
+    priceChange: float
+    priceMin: float
+    priceMax: float
+    priceYesterday: float
+    priceFirst: float
+    last: bool
+    id: int
+    insCode: str
+    dEven: int
+    hEven: int
+    pClosing: float
+    iClose: bool
+    yClose: bool
+    pDrCotVal: float
+    zTotTran: float
+    qTotTran5J: float
+    qTotCap: float
+
+
+class _ClientTypeOnDate(_TypedDict):
+    recDate: int
+    insCode: str
+    buy_I_Volume: float
+    buy_N_Volume: float
+    buy_I_Value: float
+    buy_N_Value: float
+    buy_N_Count: int
+    sell_I_Volume: float
+    buy_I_Count: float
+    sell_N_Volume: float
+    sell_I_Value: float
+    sell_N_Value: float
+    sell_N_Count: int
+    sell_I_Count: int
+
+
 class InstrumentOnDate:
 
     __slots__ = 'date', 'code', 'inst'
@@ -635,7 +872,7 @@ class InstrumentOnDate:
         self.inst = _inst
         self.code = _inst.code
 
-    async def closing_price(self) -> dict:
+    async def closing_price(self) -> _ClosingPrice:
         """Return general closing price info.
 
         Result dict has the following keys: {
@@ -659,9 +896,16 @@ class InstrumentOnDate:
         j = await _api(f'MarketData/GetInstrumentState/{self.code}/{self.date}')
         return _DataFrame(j['instrumentState'], copy=False)
 
-    async def client_types(self) -> dict:
+    async def client_type(self) -> _ClientTypeOnDate:
         return await self.inst.client_type_history(self.date)
 
+    async def client_types(self) -> dict:
+        _warn(
+            '`InstrumentOnDate.client_types` is deprecated; use `InstrumentOnDate.client_type` instead.',
+            DeprecationWarning, stacklevel=2,
+        )
+        return await self.client_type()
+
     async def holders(self) -> _DataFrame:
         """Return share/unit holders for a specific date and a day before that.
 
@@ -717,7 +961,7 @@ async def price_adjustments(flow: int) -> _DataFrame:
     return df
 
 
-async def search(skey: str, /) -> _DataFrame:
+async def old_search(skey: str, /) -> _DataFrame:
     """`skey` (search key) is usually part of the l18 or l30."""
     return _csv2df(
         _StringIO(await _get_data('search.aspx?skey=' + skey, fa=True)),
@@ -733,6 +977,65 @@ async def search(skey: str, /) -> _DataFrame:
             '_unknown1', '_unknown2', '_unknown3', '_unknown4', '_unknown5'))
 
 
+class _Search(_TypedDict):
+    insCode2: str
+    insCode3: str
+    insCode4: str
+    insCode: str
+    lVal30: str
+    lVal18AFC: str
+    flow: int
+    cIsin: None
+    zTitad: float
+    baseVol: int
+    instrumentID: None
+    cgrValCot: str
+    cComVal: None
+    lastDate: int
+    sourceID: int
+    flowTitle: str
+    cgrValCotTitle: str
+
+
+async def search(s: str, /) -> list[_Search]:
+    r = await _api(f'Instrument/GetInstrumentSearch/{s}', fa=True)
+    return r['instrumentSearch']
+
+
+class _Instrument(_TypedDict, total=False):
+    cValMne: None
+    lVal18: None
+    cSocCSAC: None
+    lSoc30: None
+    yMarNSC: None
+    yVal: None
+    insCode: str
+    lVal30: str
+    lVal18AFC: str
+    flow: int
+    cIsin: None
+    zTitad: float
+    baseVol: int
+    instrumentID: None
+    cgrValCot: None
+    cComVal: None
+    lastDate: int
+    sourceID: int
+    flowTitle: None
+    cgrValCotTitle: None
+
+
+class _ShareHolderCompany(_TypedDict):
+    instrument: _Instrument
+    numberOfShares: int
+    perOfShares: float
+
+
+async def share_holder_companies(share_holder_id: int) -> list[_ShareHolderCompany]:
+    r = await _api(f'Shareholder/GetShareHolderCompanyList/{share_holder_id}', fa=True)
+    return r['shareHolderShare']
+
+
 def _parse_price_info(price_info):
     price_parts = price_info.split(',')
 
