@@ -1,41 +1,53 @@
+from asyncio import sleep as _sleep
+from collections.abc import Callable as _Callable
+from io import BytesIO as _BytesIO, StringIO as _StringIO
+
 from numpy import nan as _nan
+from pandas import read_html as _read_html, to_numeric as _to_numeric
 
-from . import _get_data, _get_par_tree, _parse_market_state, _TypedDict, \
-    _MarketState, _csv2df, _StringIO, _BytesIO, _DataFrame, \
-    _parse_ombud_messages, _to_numeric, _read_html, _jstrptime
-
+from . import (
+    _csv2df,
+    _DataFrame,
+    _get_data,
+    _get_par_tree,
+    _jstrptime,
+    _MarketState,
+    _parse_market_state,
+    _parse_ombud_messages,
+    _TypedDict,
+)
 
 _PRICE_INDEX_COLS = ['ins_code', 'isin', 'l18', 'l30']
 _BEST_LIMITS_NAMES = ('ins_code', 'number', 'zo', 'zd', 'pd', 'po', 'qd', 'qo')
 _PRICE_DTYPES = {
-    'ins_code': 'uint64',
+    'ins_code': 'int64',
     'isin': 'string',
     'l18': 'string',
     'l30': 'string',
-    'heven': 'uint32',
-    'pf': 'uint64',
-    'pc': 'uint64',
-    'pl': 'uint64',
-    'tno': 'uint64',
-    'tvol': 'uint64',
-    'tval': 'uint64',
-    'pmin': 'uint64',
-    'pmax': 'uint64',
-    'py': 'uint64',
+    'heven': 'int32',
+    'pf': 'int64',
+    'pc': 'int64',
+    'pl': 'int64',
+    'tno': 'int64',
+    'tvol': 'int64',
+    'tval': 'int64',
+    'pmin': 'int64',
+    'pmax': 'int64',
+    'py': 'int64',
     'eps': 'float64',
-    'bvol': 'uint64',
-    'visitcount': 'uint64',
+    'bvol': 'int64',
+    'visitcount': 'int64',
     # 0-7 http://redirectcdn.tsetmc.com/Site.aspx?ParTree=1114111118&LnkIdn=83
-    'flow': 'uint8',
+    'flow': 'int16',
     # 1-98, see tsetmc.general.cs_codes()
-    'cs': 'uint8',
+    'cs': 'int16',
     'tmax': 'float64',
     'tmin': 'float64',
-    'z': 'uint64',
+    'z': 'int64',
     # 67-701 http://redirectcdn.tsetmc.com/Site.aspx?ParTree=1114111118&LnkIdn=83
-    'yval': 'uint16',
+    'yval': 'int16',
     'predtran': 'float64',
-    'buyop': 'UInt64',
+    'buyop': 'Int64',
 }
 _PRICE_COLUMNS = _PRICE_DTYPES.keys()
 _PRICE_UPDATE_COLUMNS = ('ins_code', *(*_PRICE_COLUMNS,)[4:13])
@@ -45,6 +57,7 @@ class _MarketWatchInit(_TypedDict, total=False):
     prices: _DataFrame
     best_limits: _DataFrame
     market_state: _MarketState
+    refid: int
 
 
 async def market_watch_init(
@@ -65,8 +78,8 @@ async def market_watch_init(
     """
     # todo: use content?
     text = await _get_data('MarketWatchInit.aspx?h=0&r=0', fa=True)
-    _, market_state_str, states, price_rows, _ = text.split('@')
-    result = {}
+    _, market_state_str, states, price_rows, refid = text.split('@')
+    result = {'refid': int(refid)}
     if prices:
         result['prices'] = price_df = _csv2df(
             _StringIO(states),
@@ -75,7 +88,7 @@ async def market_watch_init(
     if best_limits:
         result['best_limits'] = best_limits_df = _csv2df(
             _StringIO(price_rows), names=_BEST_LIMITS_NAMES,
-            dtype='uint64', index_col=('ins_code', 'number'))
+            dtype='int64', index_col=('ins_code', 'number'))
     if join and prices and best_limits:
         # merge multiple rows sharing the same `row` number into one row.
         # a fascinating solution from https://stackoverflow.com/a/53563551/2705757
@@ -136,13 +149,13 @@ async def market_watch_plus(
         if price_updates:
             lst = [ip for ip in inst_prices if len(ip) == 10]
             df = _DataFrame(lst, columns=_PRICE_UPDATE_COLUMNS, copy=False)
-            df = df.astype('uint64', False)
+            df = df.astype('int64', False)
             df.set_index('ins_code', inplace=True)
             result['price_updates'] = df
     if best_limits:
         result['best_limits'] = _csv2df(
             _StringIO(best_limit), index_col='ins_code',
-            names=_BEST_LIMITS_NAMES, dtype='uint64')
+            names=_BEST_LIMITS_NAMES)
     result['refid'] = int(refid)
     return result
 
@@ -172,11 +185,9 @@ async def closing_price_all() -> _DataFrame:
     """
     content = await _get_data('ClosingPriceAll.aspx')
     data = _split_id_rows(content, id_row_len=11)
-    # dtype='uint64' param cannot be used due to
-    # https://github.com/pandas-dev/pandas/issues/44835
     df = _DataFrame(data, columns=(
         'ins_code', 'n', 'pc', 'pl', 'tno', 'tvol', 'tval'
-        , 'pmin', 'pmax', 'py', 'pf'), copy=False).astype('uint64', False)
+        , 'pmin', 'pmax', 'py', 'pf'), copy=False).astype('int64')
     df.set_index(['ins_code', 'n'], inplace=True)
     return df
 
@@ -191,7 +202,7 @@ async def client_type_all() -> _DataFrame:
         _BytesIO(content), names=(
             'ins_code', 'n_buy_count', 'l_buy_count', 'n_buy_volume', 'l_buy_volume'
             , 'n_sell_count', 'l_sell_count', 'n_sell_volume', 'l_sell_volume')
-        , dtype='uint64', index_col='ins_code')
+        , dtype='int64', index_col='ins_code')
     return df
 
 
@@ -244,3 +255,65 @@ async def status_changes(top: int | str) -> _DataFrame:
         _jstrptime, format='%Y/%m/%d %H:%M:%S')
     df.drop(columns=['تاریخ', 'زمان'], inplace=True)
     return df
+
+
+class MarketWatch:
+
+    __slots__ = (
+        'interval',
+        'init_callback',
+        'init_kwargs',
+        'plus_callback',
+        'plus_kwargs',
+        'refid',
+        'heven',
+    )
+
+    def __init__(
+        self, *,
+        init_kwargs: dict=None,
+        plus_kwargs: dict=None,
+        init_callback: _Callable,
+        plus_callback: _Callable,
+        interval=1,
+    ):
+        """Create an object that helps with watching the market watch.
+
+        :param init_kwargs: kwargs to be passed to market_watch_init
+        :param plus_kwargs: kwargs to be passed to market_watch_plus
+        :param init_callback: function to be called with the result of
+            market_watch_init. This function should return True, otherwise the
+            watch will be stopped.
+        :param plus_callback: function to be called with the result of
+            market_watch_plus. This function should return True, otherwise the
+            watch will be stopped.
+        :param interval: The sleep interval.
+
+        To stop the watch, set `plus_callback` to `lambda a: False`.
+        """
+        self.interval = interval
+        self.init_kwargs: dict = {} if init_kwargs is None else init_kwargs
+        self.init_callback = init_callback
+        self.plus_kwargs: dict = {} if plus_kwargs is None else plus_callback
+        self.plus_callback = plus_callback
+
+
+    async def start(self):
+        mwi = await market_watch_init(**self.init_kwargs)
+        if not self.init_callback(mwi):
+            return
+
+        heven = mwi['prices'].heven.max()
+        refid = mwi['refid']
+
+        while True:
+            mwp = await market_watch_plus(refid=refid, heven=heven, **self.plus_kwargs)
+            if not self.plus_callback(mwp):
+                return
+            refid = mwp['refid']
+            heven = max(
+                mwp['price_updates'].heven.max(),
+                mwp['new_prices'].heven.max(),
+            )
+            print(refid, heven)
+            await _sleep(self.interval)
